package com.company;

/**
 * Класс реализующий "банк"
 */
public class Bank {
    /**
     * Голова списка счетов банка
     */
    private static Bill head;

    /**
     * Счётчик клиентов банка. Обновлятся при добавлении нового клиента или удалении старого счёта.
     */
    private static int count;

    /**
     * Наименование банка
     */
    private static String name;

    /**
     * Возвращает количество клиентов
     *
     * @return размер списка
     */
    public static int getClientCount() {
        return count;
    }

    /**
     * Возвращает наименование банка
     *
     * @return название
     */
    public static String getName() {
        return name;
    }

    /**
     * Задать наименование банка
     *
     * @param name имя банка
     */
    public static void setName(String name) {
        Bank.name = name;
    }

    /**
     * Авторизация клиента в системе
     *
     * @param bill номер счёта
     * @param pin  пин-код
     * @return true, если данные верны
     */
    public static boolean authorization(String bill, String pin) {
        Bill pointer = head;

        while (pointer != null) {
            if (pointer.getNumber().equals(bill)) {
                if (pointer.checkPincode(pin)) {
                    return true;
                }
            }

            pointer = pointer.next;
        }

        return false;
    }

    /**
     * Вставить новый элемент в конец списока (открытие нового счёта)
     *
     * @param passport серия и номер паспорта
     * @param phone    мобильный телефон
     * @param name     фамилия и имя клиента
     * @param balance  начальный баланс
     */
    public static SecretData insert(String passport, String phone, String name, double balance) {
        Bill newBill = new Bill(balance, passport, phone, name, count);

        if (head == null) {
            head = newBill;
        } else {
            Bill last = head;

            while (last.next != null) {
                last = last.next;
            }

            last.next = newBill;
        }

        count++;

        return newBill.getSecretData();
    }

    /**
     * Вставить новый элемент после указанного indexAfter (открытие ранее удалённого счёта)
     *
     * @param indexAfter индекс элемента, после которого будет вставлен новый
     * @param passport   серия и номер паспорта клиента
     * @param phone      мобильный телефон
     * @param name       фамилия и имя клиента
     * @param balance    начальный баланс
     */
    public static void insertAt(int indexAfter, String passport, String phone, String name, double balance) {
        Bill newBill = new Bill(balance, passport, phone, name, count);

        Bill current = head;
        if (indexAfter > count) {
            while (current.next != null) {
                current = current.next;
            }
        } else {
            int index = 1;

            while (index < indexAfter) {
                current = current.next;
                index++;
            }

            newBill.next = current.next;
        }
        current.next = newBill;
    }

    /**
     * Вывод информации о всех клиентах в консоль
     */
    public static void showClientsInfo() {
        Bill current = head;

        System.out.println("\nStart client list info:");
        while (current != null) {
            System.out.printf("Bill number: %s; Client: %s; Passport: %s; Phone: %s; Balance: %.2f;\n", current.getNumber(), current.getName(), current.getPassport(), current.getPhone(), current.getBalance());
            current = current.next;
        }
        System.out.println("End client list info.\n");
    }

    /**
     * Пополнить баланс (только свой счёт)
     *
     * @param value пополняемое значение
     * @param bill  пополняемый номер счёта
     */
    public static void deposit(double value, String bill) {
        Bill current = head;

        while (current != null) {
            if (bill.equals(current.getNumber())) {
                current.deposit(value);
                break;
            }
            current = current.next;
        }
    }

    /**
     * Списать баланс
     *
     * @param value списываемое значение
     * @param bill  номер счёта для списания
     */
    public static void withdraw(double value, String bill) {
        Bill current = head;

        while (current != null) {
            if (bill.equals(current.getNumber())) {
                current.withdraw(value);
                break;
            }
            current = current.next;
        }
    }

    /**
     * Разорвать договор с банком (удалить счёт)
     *
     * @param bill номер удаляемого счёта
     */
    public static void delete(String bill) {
        Bill current = head;
        Bill last = head;

        while (current != null) {
            if (bill.equals(current.getNumber())) {
                last.next = current.next;
                break;
            }

            last = current;
            current = current.next;
        }
    }
}
