package com.company;

import java.io.*;
import java.net.Socket;
import java.sql.Timestamp;

class SendMessage implements Serializable {
    public String msg;

    public SendMessage(String msg) {
        this.msg = msg;
    }
}

public class Client extends Thread {
    private Socket socket;
    private InputStream is;
    private OutputStream os;
    private boolean authorized;

    /**
     * @param socket сокет клиента
     */
    public Client(Socket socket) throws IOException {
        this.socket = socket;
        this.is = this.socket.getInputStream();
        this.os = this.socket.getOutputStream();
        authorized = false;
    }

    @Override
    public void run() {
        try {
            // Принимаем сообщение от клиента

            ObjectInputStream objectInputStream = new ObjectInputStream(is);
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(os);

            while (!isInterrupted()) {
                UserMessage userMessage = (UserMessage) objectInputStream.readObject();
                switch (userMessage.TYPE) {
                    case AUTHORIZATION:
                        System.out.println("Авторизация");
                        authorized = Bank.authorization(userMessage.BILL_NUMBER, userMessage.PIN);
                        break;
                    case SIGN_CONTRACT:
                        System.out.println("Открытие счёта");
                        SecretData secretData = Bank.insert(userMessage.PASSPORT, userMessage.PHONE, userMessage.FULLNAME, userMessage.BALANCE);
                        objectOutputStream.writeObject(secretData);
                        objectOutputStream.flush();
                        continue;
                    case BREAK_CONTRACT:
                        System.out.println("Закрытие счёта");
                        if (!authorized) {
                            System.out.println("Для совершения операций со счётом необходимо авторизоваться");
                            objectOutputStream.writeObject(new SendMessage("Действие не выполнено! Необходимо авторизоваться."));
                            objectOutputStream.flush();
                            continue;
                        }
                        Bank.delete(userMessage.BILL_NUMBER);
                        break;
                    case DEPOSIT:
                        System.out.println("Пополнение");
                        if (!authorized) {
                            System.out.println("Для совершения операций со счётом необходимо авторизоваться");
                            objectOutputStream.writeObject(new SendMessage("Действие не выполнено! Необходимо авторизоваться."));
                            objectOutputStream.flush();
                            continue;
                        }
                        Bank.deposit(userMessage.BALANCE, userMessage.BILL_NUMBER);
                        break;
                    case WITHDRAW:
                        System.out.println("Снятие");
                        if (!authorized) {
                            System.out.println("Для совершения операций со счётом необходимо авторизоваться");
                            objectOutputStream.writeObject(new SendMessage("Действие не выполнено! Необходимо авторизоваться."));
                            objectOutputStream.flush();
                            continue;
                        }
                        Bank.withdraw(userMessage.BALANCE, userMessage.BILL_NUMBER);
                        break;
                }

                // Посылаем ответ с результатом работы для не обработанных действий

                objectOutputStream.writeObject(new SendMessage("Действие успешно выполнено"));
                objectOutputStream.flush();
            }
        } catch (ClassNotFoundException | IOException exception) {

            if (exception.getMessage() != null) {
                System.out.println(exception.getMessage());
            }
            interrupt();
        }

        System.out.println(new Timestamp(System.currentTimeMillis()) + " Client disconnected: " + socket.getInetAddress().getHostAddress());

        try {
            is.close();
            os.close();
            socket.close();
        } catch (IOException exception) {
            System.out.println(exception.getMessage());
        }
    }
}
